from NewRelicApiParser.Base import BaseNewRelic
from NewRelicApiParser.CustomExceptions import ArgumentException

class Plugins(BaseNewRelic):

    def __init__(self, API_KEY):
        super().__init__(API_KEY)

    def get_list(self, options: dict = {}) -> dict:
        """
        fetch the plugins for new relic
        """
        url = self.BASE_URI + '/plugins.json'
        return super().get_data(url, options=options)

    def show(self, id: int) -> dict:
        """
        fetch single plugin based on id provided
        """
        url = self.BASE_URI + '/applcations/{0}.json'.format(id)
        return super().get_data(url)


from NewRelicApiParser.Base import BaseNewRelic
from NewRelicApiParser.CustomExceptions import ArgumentException

class KeyTransactions(BaseNewRelic):

    def __init__(self, API_KEY):
        super().__init__(API_KEY)

    def get_list(self, options: dict = {}) -> dict:
        """
        fetch the key transactions for new relic
        """
        url = self.BASE_URI + '/key_transactions.json'
        return super().get_data(url, options=options)

    def show(self, app_id: int) -> dict:
        """
        fetch single key transction data
        """
        url = self.BASE_URI + '/key_transactions/{0}.json'.format(app_id)
        return super().get_data(url)


class ArgumentException(Exception):
    message = 'Error: invalid arguments'

class InvalidKeyException(Exception):
    message = 'Invalid API key. Contact your account admin to generate a new one, or see our API docs'
    status_code = 401

class NotEnabledException(Exception):
    message = "Your New Relic API access isn't enabled. Contact your account admin, or see our API docs."
    status_code = 403

class ServerErrorException(Exception):
    message = "Something went wrong"
    status_code = 500


import requests
import logging

from NewRelicApiParser import Config

logger = logging.getLogger()

class BaseNewRelic:

    def __init__(self, API_KEY: str = None):
        self.headers = {
            "X-Api-Key": API_KEY
        }
        self.BASE_URI = Config.BASE_URI

    def get_list(self):
        """
        override with your implementation of get list
        """
        pass

    def show(self):
        """
        override with your implementation of show
        """
        pass

    def create(self):
        """
        override with your implementation of create
        """
        pass

    def update(self):
        """
        override with your implementation of update
        """
        pass

    def delete(self, url: str):
        """
        override with your implementation to delete
        """
        try:
            response = requests.delete(url)
        except Exception as ex:
            self.handle_exception(ex)
            return None

    def get_data(self, url: str, options: dict = {}):
        try:
            response = requests.get(url, headers=self.headers, data=options)
            if response.status_code == 200:
                return response.json()
            else:
                return None
        except Exception as ex:
            self.handle_exception(ex)
            return None

    def fetch_data(self):
        pass

    def post_data(self, url: str, data: dict = {}):
        try:
            response = requests.post(url, headers=self.headers, data = data)
            if response.status_code == 200:
                return response.json()
            else:
                return None
        except Exception as ex:
            self.handle_exception(ex)
            return None

    def handle_exception(self, excep):
        logger.exception("Error processing new relic request: " + str(excep))


from NewRelicApiParser.Base import BaseNewRelic

class AlertsPolicies(BaseNewRelic):

    def __init__(self, API_KEY):
        super().__init__(API_KEY)

    def get_list(self, options: dict = {}) -> dict:
        """
        fetch the alert conditions for new relic
        """
        url = self.BASE_URI + '/alerts_policies.json'
        return super().get_data(url, options=options)


from NewRelicApiParser.Base import BaseNewRelic
from NewRelicApiParser.CustomExceptions import ArgumentException

class BrowserApplications(BaseNewRelic):

    def __init__(self, API_KEY: str):
        super().__init__(API_KEY)

    def get_list(self) -> dict:
        """
        fetch the browser applications for new relic
        """
        url = self.BASE_URI + '/browser_applications.json'
        return super().get_data(url)

    def create(self, browser_application_name: str) -> dict:
        """
        create a browser application
        """
        try:
            url = self.BASE_URI + '/browser_applications.json'
            if browser_application_name is None or browser_application_name == '':
                raise ArgumentException
            data = {
                'browser_application': {
                    'name': browser_application_name
                }
            }
            return super().post_data(url, data=data)
        except ArgumentException as ae:
            return None
        except Exception as ex:
            return None


from NewRelicApiParser.Base import BaseNewRelic
from NewRelicApiParser.CustomExceptions import ArgumentException

class Users(BaseNewRelic):

    def __init__(self, API_KEY):
        super().__init__(API_KEY)

    def get_list(self, options = {}):
        """
        returns a list of User objects
        """
        url = self.BASE_URI + '/users.json'
        return super().get_data(url, options=options)

    def show(self, id, options = {}):
        """
        returns a User object corresponding to the ID provided
        """
        try:
            if id is None:
                raise ArgumentException
            url = self.BASE_URI + '/users/{0}.json'.format(id)
            return super().get_data(url, options=options)
        except ArgumentException as ae:
            print(ae)


import setuptools

def read(fname):
    """ Return file content. """
    with open(fname) as f:
        content = f.read()

    return content

description = "A plug-n-play package to start using new relic APIs for data gathering"
try:
    long_description = read('README.MD')
except IOError:
    long_description = description

setuptools.setup(
    name="newrelic-api-parser",
    package=['newrelic-api-parser'],
    version="0.2.0",
    author="Bharat Sinha",
    author_email="bharat.sinha.2307@gmail.com",
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Bharat23/newrelic-api-parser",
    packages=setuptools.find_packages(),
    license='MIT',
    keywords = ['new relic api', 'insights', 'new relic', 'apm'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
    ],
    install_requires=[
          'requests',
      ],
    python_requires='>=3.6',
)



from NewRelicApiParser.Base import BaseNewRelic
from NewRelicApiParser import Config
from urllib import parse

class Insights(BaseNewRelic):

    def __init__(self, query_key: str):
        super().__init__()
        self.BASE_URI = Config.BASE_INSIGHTS_URI
        self.headers['X-Query-Key'] = query_key

    def query(self, account_id: str, nrql: str) -> dict:
        """
        return the insights query data
        """
        url = self.BASE_URI + '/accounts/' + str(account_id) +'/query?nrql=' + parse.quote(nrql)
        return self.get_data(url)